package org.omegahat.DataStructures.Data;

import java.util.Hashtable;
import java.util.Vector;


/**
 An unordered factor variable that has a set of levels
and observed values that are indices into this set 
to identify that record's level/value.
*/
public class FactorVariable
        extends AbstractVariable
        implements FactorInt,
                   java.io.Serializable
{
  //protected Hashtable labels;
  //java.util.Vector names;

  protected FactorLevelSetInt levels;
  protected int[] levelIndices;

  protected Object[] values;

  public FactorVariable() {

  }

  public FactorVariable(int num) {
    levelIndices = new int[num];
    levels = new BasicFactorLevelSet(1);
  }


  public FactorVariable(Object[] str) {
    this(str, str.length);
  }

  public FactorVariable(Object[] str, int num) {
    this(num);

    Vector nam = new Vector(1);
    if(str != null) {
      levels(new BasicFactorLevelSet(str));
    }
  }


  public FactorVariable(Object[] str, int num, java.util.Hashtable metaData) {
   this(str, num);
   metaData(metaData);
  }

  public FactorVariable(FactorVariable var) {
    this(var.toStringArray());
  }

  
  public Object[] labels() {
    return( levels().levels());
  }

  public FactorLevelSetInt levels() 
  { 
    return(levels); 
  }

  public FactorLevelSetInt levels(FactorLevelSetInt lev) 
  { 
    levels = lev;
    return(levels()); 
  }

  public FactorLevelSetInt levels(String[] els) 
  { 
    return(levels(new BasicFactorLevelSet(els)));
  }

  public FactorLevelSetInt levels(Object value) 
  { 
    return(levels(new BasicFactorLevelSet(value)));
  }

  public FactorLevelSetInt levels(java.util.Vector v) 
  {
    return(levels(new BasicFactorLevelSet(v)));
  }

  public int numLevels() 
  { 
    return(levels.size()); 
  }


  public String toString() {
    String str = "Factor: # obs " + getSize() + " (# categories " + numLevels() + ")";

   return(str);
  }


  public int getSize() 
  { 
    return(levelIndices.length); 
  }

  public Object[] values() {
    if(values == null) {
      computeValues();
    }

   return(values);
  }

  protected Object[] computeValues() {
    int n = (int) numObservations();
    values = new Object[n];  
    Object[] tmp = levels.levels();
    for(int i = 0; i < n ; i++) {
       values[i] = tmp[levelIndices[i]];
    }
   return(values);
  }

  public Object value(int i) {
  //  return(new Integer(levelIndices[i]));
    return(levels().level(levelIndices[i]));
  }


  public Object asArray() {
    return(levels);
  }

  public Object asArrayCopy() {
    int[] l = new int[levelIndices.length];
    for(int i = 0; i < l.length; i++)
      l[i] = levelIndices[i];
    return(l);
  }

  public VariableInt values(int[] ind) {
    FactorVariable fac = new FactorVariable();
    fac.levelIndices = new int[ind.length];
    for(int i = 0; i < ind.length; i++)
      fac.levelIndices[i] = levelIndices[ind[i]];

    fac.levelIndices = levelIndices;

    return(fac);
  }

  public Object[] toStringArray() {
    int n = (int) numObservations();
    Object[] str = new Object[n];
    for(int i = 0; i < n; i++)
      str[i] = levels().elementAt(levelIndices[i]);
    return(str);
  }


  public void add(Object value, int which) {

    if(levels == null) {
        levels(value);
    }

    int level = levels().indexOf(value);
    if(level < 0) {
      levels().add(value);
      level = levels().size() - 1;
    }

    if(which < 0 || levelIndices == null) {
        int n = 0;
         if(levelIndices != null)
           n += levelIndices.length;
        int[] tmp = new int[n+1];
        if(n > 0)
          System.arraycopy(levelIndices, 0, tmp, 0, n);
        levelIndices = tmp;
        levelIndices[n] = level;

    } else
       levelIndices[which] = level;
  }

  public void add(double value, int which) { 
   add((int)value, which);
  }

  public void add(int value, int which) { 
    levelIndices[which] = value;
  }

  public long numObservations() {
    return( (levelIndices == null) ? 0 : levelIndices.length );
  }

  public double[] getValues() {
   int n = (int) numObservations();
   double[] vals = new double[n];

    for(int i = 0; i < n ; i++) 
       vals[i] = levelIndices[i];

   return(vals);
  }



  public long[] indicesForLevel(Object level) {

   return(null);
  }





}

