
package org.omegahat.DataStructures.Data;



/**
 An implementation of {@link DataFrame DataFrame}
 that provides  access to the order in which the
 variables were added.
 This uses more memory, and is implemented by embedding
 an ordered table as an internal field.
 The lack of multiple inheritance prohibits us from
 extending {@link org.omegahat.Environment.Utils.OrderedTable OrderedTable}
 and {@link DataFrame DataFrame}.
 <i>This is a rather obvious, inefficient implementation,
  acting as a place holder.</i>
  */
public class OrderedDataFrame
        extends DataFrame
{
protected org.omegahat.Environment.Utils.OrderedTable ordered;

/**
  Constructor specifying the number of records that will be in this 
  dataset.
*/
public OrderedDataFrame(Integer l) {
  this(l.intValue());
}

public OrderedDataFrame(int l) {
  this(l, null);
}    

public OrderedDataFrame(int l, org.omegahat.Environment.Databases.Database dbase) {
 super(l, dbase);
}    


/**
   Accessor for {@link #ordered ordered} field
 */
public org.omegahat.Environment.Utils.OrderedTable getOrdered()
{
   if(ordered == null)
        setOrdered(new org.omegahat.Environment.Utils.OrderedTable(size()));
   return(ordered);
}

/**
   Accessor for setting {@link #ordered ordered} field
 */

public org.omegahat.Environment.Utils.OrderedTable setOrdered(org.omegahat.Environment.Utils.OrderedTable value)
{
   ordered = value;
  return(ordered);
}



public void add(String name, VariableInt v) throws Exception {
 super.add(name, v);
 getOrdered().add(name, v);
}

public Object remove(Object obj) {
    if(obj instanceof String) {
        super.remove((String) obj);
        return(getOrdered().remove((String) obj));
    }

  return(null);
}    

public String[] getVariableNames() {
  java.util.Enumeration e = getOrdered().keys();
  int n = size();
  String[] els = new String[n];
   for(int i = 0 ; e.hasMoreElements() ; i++ )
         els[i] = (String) e.nextElement();
  return els;
} 


}

/*
Copyright (c) 1998 1999, The Omega Project for Statistical Computing.
     All rights reserved.

     Redistribution and use in source and binary forms, with or without
     modification, are permitted provided that the following conditions are
     met:

       Redistributions of source code must retain the above copyright notice,
       this list of conditions and the following disclaimer.

       Redistributions in binary form must reproduce the above copyright
       notice, this list of conditions and the following disclaimer in the
       documentation and/or other materials provided with the distribution.

       Neither name of the Omega Project for Statistical Computing nor the 
       names of its contributors may be used to endorse or promote products 
       derived from this software without specific prior written permission.

     THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ``AS IS'' AND ANY
     EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
     MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT
     SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
     SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
     PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
     INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
     STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
     THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. 

*/



